@@ -44,6 +44,16 @@
 #include "glade-clipboard.h"
 #include "glade-fixed.h"
 
+#ifdef G_OS_WIN32
+long __stdcall
+ShellExecuteA (long        hwnd,
+               const char* lpOperation,
+               const char* lpFile,
+               const char* lpParameters,
+               const char* lpDirectory,
+               int         nShowCmd);
+#endif
+
 #define GLADE_UTIL_SELECTION_NODE_SIZE 7
 #define GLADE_UTIL_COPY_BUFFSIZE       1024
 
@@ -125,23 +135,25 @@ glade_util_compose_get_type_func (const gchar *name)
 
 /**
  * glade_util_get_type_from_name:
- * @name: the name of the #GType - like 'GtkWidget'.
+ * @name: the name of the #GType - like 'GtkWidget' or a "get-type" function.
+ * @have_func: function-name flag -- true if the name is a "get-type" function.
  *
- * Looks up the type registering function from a plugin
- * and initializes & returns the type for @name.
+ * Returns the type using the "get type" function name based on @name.  
+ * If the @have_func flag is true,@name is used directly, otherwise the get-type 
+ * function is contrived from @name then used.
  *
  * Returns: the new #GType
  */
 GType
-glade_util_get_type_from_name (const gchar *name)
+glade_util_get_type_from_name (const gchar *name, gboolean have_func)
 {
 	static GModule *allsymbols = NULL;
 	GType (*get_type) ();
 	GType type = 0;
-	gchar  *func_name;
+	gchar  *func_name = (gchar*)name;
 	
 	if ((type = g_type_from_name (name)) == 0 &&
-	    (func_name = glade_util_compose_get_type_func (name)) != NULL)
+	    (have_func || (func_name = glade_util_compose_get_type_func (name)) != NULL))
 	{
 		
 		if (!allsymbols)
@@ -1199,14 +1211,14 @@ glade_util_purify_list (GList *list)
  *
  */
 GList *
-glade_util_added_in_list (GList *old,
-			  GList *new)
+glade_util_added_in_list (GList *old_list,
+			  GList *new_list)
 {
 	GList *added = NULL, *list;
 
-	for (list = new; list; list = list->next)
+	for (list = new_list; list; list = list->next)
 	{
-		if (!g_list_find (old, list->data))
+		if (!g_list_find (old_list, list->data))
 			added = g_list_prepend (added, list->data);
 	} 
 
@@ -1223,14 +1235,14 @@ glade_util_added_in_list (GList *old,
  *
  */
 GList *
-glade_util_removed_from_list (GList *old,
-			      GList *new)
+glade_util_removed_from_list (GList *old_list,
+			      GList *new_list)
 {
 	GList *added = NULL, *list;
 
-	for (list = old; list; list = list->next)
+	for (list = old_list; list; list = list->next)
 	{
-		if (!g_list_find (new, list->data))
+		if (!g_list_find (new_list, list->data))
 			added = g_list_prepend (added, list->data);
 	} 
 
@@ -1755,3 +1767,173 @@ glade_util_object_is_loading (GObject *object)
 	
 	return glade_project_is_loading (project);
 }
+
+#ifdef G_OS_WIN32
+#define SW_NORMAL 1
+
+static gboolean
+glade_util_url_show_win32 (const gchar *url)
+{
+	gint retval;
+	/* win32 API call */
+	retval = ShellExecuteA (NULL, "open", url, NULL, NULL, SW_NORMAL);
+	
+	if (code <= 32)
+		return FALSE;
+
+	return TRUE;
+}
+
+#else
+
+/* pilfered from Beast - birnetutils.cc */
+static gboolean
+glade_util_url_show_unix (const gchar *url)
+{
+	static struct {
+	const gchar   *prg, *arg1, *prefix, *postfix;
+	gboolean       asyncronous; /* start asyncronously and check exit code to catch launch errors */
+	volatile gboolean disabled;
+	} browsers[] = {
+
+	/* configurable, working browser launchers */
+	{ "gnome-open",             NULL,           "", "", 0 }, /* opens in background, correct exit_code */
+	{ "exo-open",               NULL,           "", "", 0 }, /* opens in background, correct exit_code */
+
+	/* non-configurable working browser launchers */
+	{ "kfmclient",              "openURL",      "", "", 0 }, /* opens in background, correct exit_code */
+	{ "gnome-moz-remote",       "--newwin",     "", "", 0 }, /* opens in background, correct exit_code */
+
+#if 0   /* broken/unpredictable browser launchers */
+	{ "browser-config",         NULL,            "", "", 0 }, /* opens in background (+ sleep 5), broken exit_code (always 0) */
+	{ "xdg-open",               NULL,            "", "", 0 }, /* opens in foreground (first browser) or background, correct exit_code */
+	{ "sensible-browser",       NULL,            "", "", 0 }, /* opens in foreground (first browser) or background, correct exit_code */
+	{ "htmlview",               NULL,            "", "", 0 }, /* opens in foreground (first browser) or background, correct exit_code */
+#endif
+
+	/* direct browser invocation */
+	{ "x-www-browser",          NULL,           "", "", 1 }, /* opens in foreground, browser alias */
+	{ "firefox",                NULL,           "", "", 1 }, /* opens in foreground, correct exit_code */
+	{ "mozilla-firefox",        NULL,           "", "", 1 }, /* opens in foreground, correct exit_code */
+	{ "mozilla",                NULL,           "", "", 1 }, /* opens in foreground, correct exit_code */
+	{ "konqueror",              NULL,           "", "", 1 }, /* opens in foreground, correct exit_code */
+	{ "opera",                  "-newwindow",   "", "", 1 }, /* opens in foreground, correct exit_code */
+	{ "epiphany",               NULL,           "", "", 1 }, /* opens in foreground, correct exit_code */	
+	{ "galeon",                 NULL,           "", "", 1 }, /* opens in foreground, correct exit_code */
+	{ "amaya",                  NULL,           "", "", 1 }, /* opens in foreground, correct exit_code */
+	{ "dillo",                  NULL,           "", "", 1 }, /* opens in foreground, correct exit_code */
+
+	};
+  
+	guint i;
+	for (i = 0; i < G_N_ELEMENTS (browsers); i++)
+
+		if (!browsers[i].disabled)
+		{
+		        gchar *args[128] = { 0, };
+		        guint n = 0;
+		        args[n++] = (gchar*) browsers[i].prg;
+		        
+		        if (browsers[i].arg1)
+		        	args[n++] = (gchar*) browsers[i].arg1;
+		        
+		        gchar *string = g_strconcat (browsers[i].prefix, url, browsers[i].postfix, NULL);
+		        args[n] = string;
+		        gchar fallback_error[64] = "Ok";
+		        gboolean success;
+        
+		        if (!browsers[i].asyncronous) /* start syncronously and check exit code */
+			{
+				gint exit_status = -1;
+				success = g_spawn_sync (NULL, /* cwd */
+		                                        args,
+		                                        NULL, /* envp */
+		                                        G_SPAWN_SEARCH_PATH,
+		                                        NULL, /* child_setup() */
+		                                        NULL, /* user_data */
+		                                        NULL, /* standard_output */
+		                                        NULL, /* standard_error */
+		                                        &exit_status,
+		                                        NULL);
+		                success = success && !exit_status;
+		            
+				if (exit_status)
+					g_snprintf (fallback_error, sizeof (fallback_error), "exitcode: %u", exit_status);
+
+			}
+		        else
+			{
+				success = g_spawn_async (NULL, /* cwd */
+							 args,
+							 NULL, /* envp */
+							 G_SPAWN_SEARCH_PATH,
+							 NULL, /* child_setup() */
+							 NULL, /* user_data */
+							 NULL, /* child_pid */
+							 NULL);
+			}
+			
+			g_free (string);
+			if (success)
+				return TRUE;
+			browsers[i].disabled = TRUE;
+	}
+	
+	/* reset all disabled states if no browser could be found */
+	for (i = 0; i < G_N_ELEMENTS (browsers); i++)
+		browsers[i].disabled = FALSE;
+		     
+	return FALSE;	
+}
+
+#endif
+
+/**
+ * glade_util_url_show:
+ * @url: An URL to display
+ *
+ * Portable function for showing an URL @url in a web browser.
+ *
+ * Returns: TRUE if a web browser was successfully launched, or FALSE
+ * 
+ */
+gboolean
+glade_util_url_show (const gchar *url)
+{
+	g_return_val_if_fail (url != NULL, FALSE);
+
+#ifdef G_OS_WIN32
+	return glade_util_url_show_win32 (url);
+#else
+	return glade_util_url_show_unix (url);
+#endif
+}
+
+/**
+ * glade_util_get_file_mtime:
+ * @filename: A filename
+ * @error: return location for errors
+ *
+ * Gets the UTC modification time of file @filename.
+ *
+ * Returns: The mtime of the file, or %0 if the file attributes
+ *          could not be read.
+ */
+time_t
+glade_util_get_file_mtime (const gchar *filename, GError **error)
+{
+	struct stat info;
+	gint retval;
+	
+	retval = g_stat (filename, &info);
+	
+	if (retval != 0) {
+		g_set_error (error,
+		             G_FILE_ERROR,
+		             g_file_error_from_errno (errno),
+		             "could not stat file '%s': %s", filename, g_strerror (errno));    
+		return (time_t) 0;
+	} else {
+		return info.st_mtime;
+	}
+}
