@@ -28,7 +28,6 @@
 typedef struct _GladeWidget         GladeWidget;
 typedef struct _GladeProperty       GladeProperty;
 typedef struct _GladeProject        GladeProject;
-typedef enum   _GladeItemAppearance GladeItemAppearance;
 
 #include "glade-widget-adaptor.h"
 #include "glade-widget.h"
@@ -150,6 +149,7 @@ typedef enum   _GladeItemAppearance GladeItemAppearance;
 #define GLADE_TAG_ACTION_GROUP                    "action-group"
 #define GLADE_TAG_ACTION                          "action"
 #define GLADE_TAG_STOCK                           "stock"
+#define GLADE_TAG_GET_TYPE_FUNCTION               "get-type-function"
 
 #define GLADE_NUMERICAL_STEP_INCREMENT             1
 #define GLADE_FLOATING_STEP_INCREMENT              0.01F
